;
; 1 JUL 80
;
	TITLE	'CCS 2422 STANDARD BIOS FOR CP/M 2.2'
	PAGE	56
	MACLIB	Z80
;
; THIS BIOS IS SET UP FOR STANDARD 128 BYTE SECTORS ONLY
;
;"BIAS" IS ADDRESS OFFSET FROM 3400H FOR MEMORY SYSTEMS
;THAN 20K (REFERRED TO AS "B" THROUGHOUT THE TEXT).
;
VERS:	EQU	22	;CP/M VERSION NUMBER
MSIZE:	EQU	20	;CP/M VERSION MEMORY SIZE IN KILOBYTES
BIAS:	EQU	(MSIZE-20)*1024
CCP:	EQU	2C00H+BIAS	;BASE OF CCP
BDOS:	EQU	CCP+806H	;BASE OF BDOS
BIOS:	EQU	CCP+1600H	;BASE OF BIOS
;
SDATA:	EQU	20H	;SERIAL DATA PORT
SINTEN:	EQU	SDATA+1	;SERIAL INTERRUPT ENABLE PORT
SIDENT:	EQU	SDATA+2	;SERIAL INTERRUPT IDENTIFICATION PORT
SLCTRL:	EQU	SDATA+3	;SERIAL LINE CONTROL PORT
SMDMCT:	EQU	SDATA+4	;SERIAL MODEM CONTROL PORT
SLSTAT:	EQU	SDATA+5	;SERIAL LINE STATUS PORT
SMDMST:	EQU	SDATA+6	;SERIAL MODEM STATUS PORT
;
DSTAT	EQU	30H	;DISK STATUS PORT
DCMMD	EQU	DSTAT	;DISK COMMAND PORT
DTRCK	EQU	DSTAT+1	;DISK TRACK PORT
DSCTR	EQU	DSTAT+2	;DISK SECTOR PORT
DDATA	EQU	DSTAT+3	;DISK DATA PORT
DFLAG	EQU	DSTAT+4	;DISK FLAF PORT
DCNTL	EQU	DSTAT+4	;DISK CONTROL PORT
;
WSVEC:	EQU	0	;VECTOR FOR WARM RESTART
CR:	EQU	0DH	;ASCII CARRIAGE RETURN
LF:	EQU	0AH	;ASCII LINE FEED
IOBYTE:	EQU	3	;ADDRESS OF I/O CONTROL BYTE
;
DISKNO:	EQU	40H	;ACTIVE DISK NUMBER
TRACK:	EQU	DISKNO+1
SECTOR:	EQU	TRACK+1
SIDE:	EQU	SECTOR+1  ;SIDE SELECT HOLD AREA
SPT:	EQU	SIDE+1	;SECTORS PER TRACK HOLD
TWOSID:	EQU	SPT+1	;SINGLE/DOUBLE SIDED SWITCH HOLD
STPRAT:	EQU	46H	;STEP RATE SAVE AREA
STATUS:	EQU	47H
CMND:	EQU	STATUS+1
LUNIT:	EQU	49H	;LAST USED DRIVE
CUNIT:	EQU	LUNIT+1	;CURRENT DRIVE
RWFLG:	EQU	4BH
DMAAD:	EQU	4CH	;HOST BUFFER ADDRESS
IDSV:	EQU	4EH	;DISK ID SAVE AREA
TBUF:	EQU	80H
;
	ORG	BIOS	;ORIGIN OF THIS PROGRAM
;
;JUMP VECTOR FOR INDIVIDUAL SUBROUTINES
	JMP	BOOT	;COLD START
WBOOTE:	JMP	WBOOT	;WARM START
CSTAT:	JMP	TTST	;CONSOLE STATUS
CONIN:	JMP	TTYIN	;CONSOLE CHARACTER IN
CONO:	JMP	TTYOUT	;CONSOLE CHARACTER OUT
	JMP	TTYOUT	;LIST CHARACTER OUT
	JMP	TTYOUT	;PUNCH CHARACTER OUT
	JMP	TTYIN	;READER CHARACTER OUT
	JMP	HOME	;MOVE HEAD TO HOME POSITION
	JMP	SELDSK	;SELECT DISK
	JMP	SETTRK	;SET TRACK NUMBER
	JMP	SETSEC	;SET SECTOR NUMBER
	JMP	SETDMA	;SET DMA ADDRESS
	JMP	DREAD	;READ DISK
	JMP	DWRITE	;WRITE DISK
	JMP	TTOST	;RETURN LIST STATUS
	JMP	SECTRAN	;SECTOR TRANSLATE
;
;INDIVIDUAL SUBROUTINES TO PERFORM EACH FUNCTION
;
; I/O DRIVERS FOR THE 8250 ASYNC COMM ELEMENT
;
TTST:	IN	SLSTAT	;GET 8250 LINE STATUS
	ANI	1	;SEE IF RECEIVE DATA AVAILABLE
	RZ		;RETURN IF NOT
	ADI	0FEH	;FLAG THAT DATA IS AVAILABLE
	RET
;
TTYIN:	IN	SLSTAT	;GET 8250 LINE STATUS
	RAR		;MOVE RX DATA READY BIT INTO CARRY
	JRNC	TTYIN	;LOOP UNTIL DATA IS IN
	IN	SDATA	;READ THE DATA
	ANI	7FH	;STRIP OFF THE PARITY BIT
	RET
;
TTOST:	IN	SLSTAT	;GET 8250 LINE STATUS
	ANI	20H	;ISOLATE TX BUFFER EMPTY BIT
	RZ		;RETURN IF NOT EMPTY
	ADI	0BFH	;FLAG THE EMPTY STATE
	RET
;
TTYOUT:	CALL	TTOST	;GET 8250 LINE STATUS
	JRZ	TTYOUT	;WAIT UNTIL ONE OF THE REGISTERS EMPTIES
	MOV	A,C	;MOVE THE DATA OVER
	OUT	SDATA	;OUTPUT THE DATA
	RET
;
;
RETRY:	DCR	C
	JNZ	LOAD1
	LXI	H,BOTMSG
	CALL	PRTWD
	CALL	CONIN
	CALL	PRTWD
WBOOT:	LXI	SP,80H
	LDA	4	;GET OLD DEFAULT DRIVE
	STA	CDISK
	MVI	C,10	;# OF RETRIES
LOAD1:	XRA	A
	MOV	H,A	;SET THE UNIT AND TRACK
	MOV	L,A
	SHLD	DISKNO
	LXI	H,0D002H  ;SET THE SIDE AND SECTOR
	SHLD	SECTOR
	MVI	B,8	;ZERO OUT PRMTBL
	LXI	H,PRMTBL
LOAD1A:	MOV	M,A
	INX	H
	DCR	B
	JNZ	LOAD1A
	PUSH	B
	MOV	C,A
	CALL	SELDSK
	POP	B
	LDA	CUNIT	;GET THE SELBITS
	ANI	10H	;SEE IF A MINI
	MVI	E,26	;SET 8" SECTORS PER TRACK
	JNZ	LOAD1B	;JUMP IF 8"
	MVI	E,18	;  ELSE, SET MINI SECTORS PER TRACK
LOAD1B:	LXI	H,CCP
	SHLD	DMAAD
LOAD2:	PUSH	B
	PUSH	D
	CALL	DREAD
	POP	D
	POP	B
	ORA	A
	JNZ	RETRY
	SHLD	DMAAD
	LDA	SECTOR
	CMP	E
	JC	LOAD3
	LDA	TRACK
	INR	A
	STA	TRACK
	XRA	A
LOAD3:	INR	A
	STA	SECTOR
	MOV	A,H
	CPI	BIOS/256  ;SEE IF DONE WITH WARM BOOT
	JC	LOAD2	;JUMP IF MORE TO GO
;
;END OF LOAD OPERATION, SET PARAMETERS AND GO TO CP/M
BOOT0:	LXI	H,TPLT	;MOVE DOWN THE COMMON JUMP VECTORS
	LXI	D,0
	MVI	B,8
BOOT1:	MOV	A,M
	STAX	D
	INX	H
	INX	D
	DCR	B
	JNZ	BOOT1
	LXI	H,TBUF	;SET UP DEFAULT BUFFER ADDRESS
	SHLD	DMAAD
	EI		;ENABLE THE INTERRUPT SYSTEM
	LDA	CDISK	;GET CURRENT DISK NUMBER
	MOV	C,A	;SEND TO THE CCP
	JMP	CCP	;GO TO CP/M FOR FURTHER PROCESSING
;
BOOT:	LXI	H,LOGMSG  ;SIGN ON TO THE SYSTEM
	CALL	PRTWD
	JMP	BOOT0
;
TPLT:	JMP	WBOOTE
IOBYT:	DB	0
CDISK:	DB	0
	JMP	BDOS
;
;
;SELECT DISK GIVEN BY REGISTER C
SELDSK:	LXI	H,0	;ERROR RETURN CODE
	MOV	A,C
	STA	DISKNO
	CPI	4	;MUST BE BETWEEN 0 AND 3
	RNC		;NO CARRY IF 4,5,...
;DISK NUMBER IS IN THE PROPER RANGE
;COMPUTE PROPER DISK PARAMETER HEADER ADDRESS
SELDSK1:  MOV	L,A	;L=DISK NUMBER 0,1,2,3
	DAD	H	;*2
	DAD	H	;*4
	DAD	H	;*8
	DAD	H	;*16 (SIZE OF EACH HEADER)
	XCHG
	LXI	H,DPBASE
	DAD	D	;HL=.DPBASE(DISKNO*16)
;
CKSET:	PUSH	H	;SAVE (H,L)
	MOV	A,C	;REGET UNIT
	ADD	A	;OFFSET INTO TABLE
	LXI	H,PRMTBL+1  ;POINT TO SELBITS
	MOV	E,A
	DAD	D	;OFFSET THE TABLE POINTER
	MOV	A,M	;GET THE SELBITS
	ORA	A
	STA	CUNIT
	ORA	A
	JNZ	SET4	;JUMP IF ALREADY ESTABLISHED
	XCHG		;ELSE, ESTABLISH NEW DISK
	INR	L	;INSURE LUNIT<>CUNIT
	SHLD	LUNIT	;  TO FORCE A READ ADDRESS
	PUSH	B
	PUSH	D	;SAVE THE REGISTERS
	CALL	IDRD	;FIND OUT WHAT IS OUT THERE
	POP	H	;REGET SELECT TABLE ADDRESS
	POP	B
	DCX	H	;POINT TO STEP RATE
	LDA	STPRAT	;SET THE STEP RATE
	MOV	M,A	;SAVE THE STEP RATE
	INX	H
	LDA	CUNIT	;GET THE SELBITS
	MOV	M,A	;SET THE TABLE
	ANI	10H	;ISOLATE THE MINI DRIVE BIT
	LXI	D,MSELTB  ;SET ADDRESS FOR MINI
	JZ	SETUP1	;JUMP IF MINI
	LXI	D,SELTBL  ;  ELSE, SET THE 8" TABLE ADDRESS
SETUP1:	POP	H
	PUSH	H	;GET, RESAVE DP BLOCK POINTER
	LDAX	D	;MOVE THE TABLE ENTRIES
	MOV	M,A
	INX	D
	INX	H
	LDAX	D
	MOV	M,A
	INX	D
	PUSH	D	;SAVE IT FOR A MOMENT
	LXI	D,9
	DAD	D	;OFFSET THE POINTER
	POP	D	;REGET TABLE ADDRESS
	LDAX	D	;MOVE THE TABLE ENTRIES
	MOV	M,A
	INX	D
	INX	H
	LDAX	D
	MOV	M,A
SET4:	POP	H
	MOV	A,C
	RET
;
;SET TRACK GIVEN BY REGISTER C
SETTRK:	MOV	A,C
	STA	TRACK
	RET
;
;SET SECTOR GIVEN BY REGISTER C
SETSEC:	MVI	A,0D0H	;SET UP SIDE SELECT BITS
	STA	SIDE	;SET THE SIDE SELECT BITS
	MOV	A,C
	STA	SECTOR
	RET
;
;TRANSLATE THE SECTOR GIVEN BY BC USING THE
;TRANSLATE TABLE GIVEN BY DE
SECTRAN:  XCHG		;HL=.TRANS
	DAD	B	;HL=.TRANS(SECTOR)
	MOV	L,M	;L = TRANS(SECTOR)
	MVI	H,0	;HL= TRANS(SECTOR)
	RET		;WITH VALUE IN HL
;
;SET DMA ADDRESS GIVEN BY REGISTERS B AND C
SETDMA:	MOV	L,C	;LOW ORDER ADDRESS
	MOV	H,B	;HIGH ORDER ADDRESS
	SHLD	DMAAD	;SAVE THE ADDRESS
	RET
;
;MOVE TO THE TRACK 00 POSITION OF CURRENT DRIVE
HOME:	SUB	A
	STA	TRACK
	RET
;
; ROUTINE PRTWD PRINTS AN ASCII STRING ONTO THE CONSOLE.
;	THE STRING MUST BE TERMINATED BY BIT 7 SET IN THE
;	LAST CHARACTER OF THE STRING.  THE STRING WILL START
;	A NEW LINE (EP = PRTWD) OR CONTINUE ON THE SAME
;	LINE (EP = PRTWA)
;
PRTWD:	CALL	CRLF	;START A NEW LINE
PRTWA:	PUSH	B	;SAVE (B,C)
PRTA:	MOV	C,M	;GET NEXT CHARACTER FROM MEMORY
	CALL	CONO	;OUTPUT IT
	INX	H	;INCREMENT MEMORY POINTER
	MOV	A,C
	RLC		;TEST FOR BIT 7 DELIMITER
	JRNC	PRTA	;NO DELIMITER, GO DO NEXT CHARACTER
PRTB:	POP	B	;RESTORE (B,C)
	RET
;
; ROUTINE CRLF GENERATES A CARRIAGE RETURN, LINE FEED
;	SEQUENCE ON THE CURRENT CONSOLE TO START A NEW LINE
;	IT INCLUDES TWO NULL CHARACTERS FOR TTY TYPE
;	DEVICES FOR THE HEAD MOVEMENT TIME.
;
CRLF:	PUSH	H	;SAVE THE CONTENTS OF (H,L)
CRLFA:	LXI	H,CRMSG	;ADDRESS OF CR,LF MESSAGE
	CALL	PRTWA	;  OUTPUT IT
	POP	H	;RESTORE (H,L)
	RET
CRMSG:	DB	CR,LF,0,80H
;
;
; THE FOLLOWING ROUTINES DO THE PRIMITIVE DISK ACCESSES.
;	IN ALL CASES, ONE SECTOR OF DATA IS TRANSFERRED.
;	IF THE DISK HAS NOT BEEN PREVIOUSLY ACCESSED,
;	THESE ROUTINES WILL AUTOMATICALLY DETERMINE THE
;	DISK TYPE (8" OR 5"), SINGLE OR DOUBLE DENSITY,
;	AND SECTOR SIZE.
;
;	BEFORE THE DESIRED DATA IS TRANSFERRED, THE DESIRED
;	TRACK IS SEEKED OUT, THE DESIRED SECTOR AND SIDE IS
;	SET, THEN THE ACTUAL DATA TRANSFER.
;
;	UP TO TEN TRIES WILL BE ATTEMPTED BEFORE THE DATA
;	TRANSFER IS ABORTED.  ON RETURN TO THE CALLING
;	ROUTINE, THE A REGISTER WILL CONTAIN A ZERO IF THE
;	OPERATION WAS SUCCESSFUL, OR NON-ZERO IF NOT
;	SUCCESSFUL.  THE FLAG REGISTER WILL NOT NECESSARILY
;	CORRESPOND WITH THE A REGISTER CONTENT.
;
;	THESE ROUTINES ARE CP/M COMPATABLE, AND MAY BE USED
;	AS PART OF THE BIOS.
;
;
DREAD:	MVI	A,1	;SET READ FLAG
	ORG	$-1	;SAVE A BYTE HERE
DWRITE:	XRA	A	;SET WRITE FLAG
	STA	RWFLG	;SAVE IT FOR LATER USE
	MVI	B,10	;NUMBER OF RETRIES
AGN:	PUSH	B
	CALL	SEEK
	CZ	RDWR
READ3:	POP	B
	RZ
	DJNZ	AGN
	RET
;
RDWR:	MOV	E,A	;SAVE COMMAND
	LDA	RWFLG
	ORA	A
	MOV	A,E	;REGET THE COMMAND
	JRZ	WRDAT	;WRITE IF ZERO
RDAT:	OUT	DCMMD	;DISK COMMAND PORT
	STA	CMND
READ1:	IN	DDATA	;GET THE DATA
	MOV	M,A	;STORE IT
	INX	H	;INCREMENT THE MEMORY POINTER
	DCR	C	;LOOP CONTROL
	JRNZ	READ1
	DCR	B
	JRNZ	READ1
	CALL	EOJ
	ANI	9CH	;ISOLATE READ ERROR BITS
	RET
;
WRDAT:	ORI	20H	;ADD WRITE COMMAND
	OUT	DCMMD	;DISK COMMAND PORT
	STA	CMND
WRT1:	MOV	A,M	;GET THE DATA
	OUT	DDATA
	INX	H	;BUMP THE MEMORY POINTER
	DCR	C	;LOOP CONTROL
	JRNZ	WRT1
	DCR	B	;IN CASE > 256 BYTES
	JRNZ	WRT1
	JMPR	EOJ
;
EOJB:	MVI	B,8	;BASIS OF RESTORE COMMAND
EOJA:	LDA	STPRAT	;GET THE STEP RATE BITS
	ORA	B	;ADD ON THE COMMAND
	STA	CMND
	OUT	DCMMD	;DO THE COMMAND
EOJ:	IN	DFLAG	;DISK FLAG PORT
	RAR
	JRNC	EOJ
EOJ1:	IN	DSTAT	;GET THE DISK STATUS
	STA	STATUS
	ANI	0FCH
	RET
;
SEEK:	CALL	IDRD	;INSURE HEADER HAS BEEN READ
	CNZ	EOJB	;RESTORE THE DRIVE IF ERROR
	RM		;DONE IF NO DRIVE
SEEK1:	LDA	SECTOR	;SET THE SECTOR
	OUT	DSCTR	;DISK SECTOR PORT
	IN	DTRCK	;DISK TRACK PORT
	MOV	C,A	;SAVE IT
	LDA	TRACK	;GET DESIRED TRACK
	CMP	C
	JRZ	RDWRT	;JUMP IF NO SEEK NEEDED
	OUT	DDATA	;SET THE SEEK TRACK
	MVI	B,1CH	;BUILD THE SEEK COMMAND
	CALL	EOJA	;DO THE SEEK
	ANI	98H	;SEEK ERROR MASK
	RNZ		;DONE IF SEEK ERROR
	IN	DTRCK	;CHECK FOR TRACK 00
RDWRT:	ORA	A
	LXI	H,40H	;BUILD SECTOR BYTE COUNT
	JRZ	RDWRT0	;JUMP IF TRACK 00
	LDA	IDSV+3	;GET SECTOR SIZE
RDWRT0:	DAD	H	;DOUBLE (H,L)
	DCR	A	;LOOP CONTROL
	JP	RDWRT0
	PUSH	H
	MVI	C,80H	;AUTO-WAIT BIT
	CALL	SETUP
	IN	DFLAG	;DISK FLAG PORT
	ANI	20H	;SEE IF HEAD IS LOADED
	MVI	A,4
	JRZ	RDWRT1	;JUMP IF NOT
	XRA	A	;ELSE, RESET THE HEAD LOAD FLAG
RDWRT1:	ADI	88H	;BUILD A READ SECTOR COMMAND
	LHLD	DMAAD	;GET THE DMA ADDRESS
	POP	B	;GET THE BYTE COUNT
	DCR	B	;SEE IF 128 BYTE SECTOR
	INR	B
	JRNZ	RDWRT2	;JUMP IF NOT
	INR	B
RDWRT2:	CMP	A	;CLEAR THE FLAGS
	RET
;
IDRD5:	MVI	B,58H	;BUILD A STEP-IN COMMAND
	CALL	EOJA
IDRD:	LHLD	LUNIT
	MOV	A,H	;GET THE CUNIT VALUE
	CMP	L	;SEE IF SAME AS LUNIT
	RZ		;RETURN IF SO
IDRD1:	MVI	C,80H	;SET THE AUTO-WAIT BIT
	CALL	SETUP
	CALL	EOJ1	;INSURE A DRIVE IS THERE
	RM		;ERROR IF NOT
	PUSH	H	;SAVE POINTER
	LXI	H,IDSV	;SET UP TO READ ADDRESS
	LXI	B,106H	;READ 6 BYTES OF DATA
	MVI	A,0C4H	;READ ADDRESS COMMAND
	CALL	RDAT
	POP	H	;RESTORE POINTER
	JRZ	IDRD2	;JUMP IF GOOD READ
	MVI	A,40H	;SEE IF DDEN IS SET
	CMP	M
	RC		;TAKE THE ERROR IF SO
	ORA	M	;ELSE, TRY DDEN
	MOV	M,A
	JMPR	IDRD
;
IDRD2:	IN	DSCTR	;GET THE TRACK NUMBER
	OUT	DTRCK	;SET THE TRACK REGISTER
	ORA	A	;INSURE NOT ON TRACK 0
	JRZ	IDRD5	;JUMP IF NOT OKAY
	MOV	A,M	;REGET SELBITS
	STA	LUNIT	;UPDATE LAST USED UNIT
	XRA	A	;RESET ERROR FLAGS
	RET
;
;SET UP DRIVE NUMBER
SETUP:	LXI	H,CUNIT	;SEE IF DRIVE HAS BEEN ACTIVE
	MOV	A,M	;GET THE SELBITS
	ORA	A	;SEE IF SET UP YET
	JRNZ	SU0	;YES, SKIP INIT CODE
;
SETIT:	LDA	DISKNO	;GET THE DESIRED DRIVE
	MOV	B,A	;SAVE IN WORK REGISTER
	INR	B	;PREPARE TO CONVERT TO SELBITS
	XRA	A	;ZERO TO A
	STC		;DRIVE SELECT BIT
SET1:	RAL		;SHIFT BIT INTO POSITION
	DJNZ	SET1	;LOOP TIL BIT IS IN POSITION
	ORI	20H	;ADD ON MOTOR ON BIT
	MOV	M,A	;SAVE IT
	OUT	DCNTL	;SELECT THE DRIVE
	CALL	EOJB	;RESTORE THE DRIVE
	RM		;DONE IF DRIVE NOT READY
	IN	4	;READ THE MINI TRK00 BIT
	RAR		;ISOLATE IT
	MVI	A,3	;MINI STEP RATE BITS
	JRNC	SET2	;JUMP IF MINI DRIVE
	MVI	A,10H	;ELSE, ADD ON MAXI BIT
	ORA	M
	MOV	M,A
	MVI	A,2	;SET MAXI STEP RATE
SET2:	STA	STPRAT
SU0:	IN	DTRCK	;ELSE, SEE IF TRACK ZERO
	ORA	A
	IN	DDATA	;CLEAR THE STATUS BITS
	MOV	A,M	;REGET THE SELBITS
	JRNZ	SU1
	ANI	0BFH	;INSURE DDEN IS RESET
SU1:	ORA	C	;ADD ON AUTOWAIT BIT
	OUT	DCNTL	;OUTPUT THE SELBITS
	LDA	SIDE	;SET THE SIDE SELECT
	OUT	4
	RET
;
BOTMSG:	DB	7,'CANNOT BOO','T'+80H,0DH,8AH
LOGMSG:	DB	MSIZE/10+'0',MSIZE MOD 10 + '0'
	DB	'k CP/M vers '
	DB	VERS/10+'0','.',VERS MOD 10+'0'
	DB	0DH,8AH
;
;
;FIXED DATA TABLES FOR FOUR-DRIVE STANDARD
;IBM-COMPATIBLE 8" DISKS
;DISK PARAMETER HEADER FOR DISK 00
DPBASE:	DW	0,0
	DW	0,0
	DW	DIRBF,0
	DW	CHK00,ALL00
;DISK PARAMETER HEADER FOR DISK 01
	DW	0,0
	DW	0,0
	DW	DIRBF,0
	DW	CHK01,ALL01
;DISK PARAMETER HEADER FOR DISK 02
	DW	0,0
	DW	0,0
	DW	DIRBF,0
	DW	CHK02,ALL02
;DISK PARAMETER HEADER FOR DISK 03
	DW	0,0
	DW	0,0
	DW	DIRBF,0
	DW	CHK03,ALL03
;
;PARAMETER TABLE FOR DRIVE-UNIQUE CONSTANTS
PRMTBL:	DB	0,0	;DRIVE 0 STEP RATE, SELECT BYTES
	DB	0,0	;      1
	DB	0,0	;      2
	DB	0,0	;      3
;
SELTBL:	DW	T8S0,DP8S0
DP8S0:	DW	26	;SECTORS PER TRACK
	DB	3	;BLOCK SHIFT FACTOR
	DB	7	;BLOCK MASK
	DB	0	;EXTENT MASK
	DW	242	;BLOCKS PER DISKETTE
	DW	63	;# DIRECTORY ENTRIES
	DB	192	;ALLOC 0
	DB	0	;ALLOC 1
	DW	16	;DIR CHECK VECTOR SIZE
	DW	2	;SYSTEM TRACK OFFSET
T8S0:	DB	1,7,13,19,25
	DB	5,11,17,23
	DB	3,9,15,21
	DB	2,8,14,20,26
	DB	6,12,18,24
	DB	4,10,16,22
;
MSELTB: DW	T5S0,DP5S0
DP5S0:	DW	18	;SECTORS PER TRACK
	DB	3	;BLOCK SHIFT FACTOR
	DB	7	;BLOCK MASK
	DB	0	;EXTENT MASK
	DW	71	;BLOCKS PER DISKETTE
	DW	63	;# DIRECTORY ENTRIES
	DB	192	;ALLOC 0
	DB	0	;ALLOC 1
	DW	16	;DIR CHECK VECTOR SIZE
	DW	3	;SYSTEM TRACK OFFSET
T5S0:	DB	1,5,9,13,17
	DB	3,7,11,15
	DB	2,6,10,14,18
	DB	4,8,12,16
;
;END OF FIXED TABLES
;
;
;THE REMAINDER OF THE CBIOS IS RESERVED UNINITIALIZED
;DATA AREA, AND DOES NOT NEED TO BE A PART OF THE
;SYSTEM MEMORY IMAGE (THE SPACE MUST BE AVAILABLE,
;HOWEVER, BETWEEN "BEGDAT" AND "ENDDAT").
;
;
;SCRATCH RAM AREA FOR BDOS USE
BEGDAT	EQU	$	;BEGINNING OF DATA AREA
DIRBF:	DS	128	;SCRATCH DIRECTORY AREA
ALL00:	DS	31	;ALLOCATION VECTOR 0
ALL01:	DS	31	;ALLOCATION VECTOR 1
ALL02:	DS	31	;ALLOCATION VECTOR 2
ALL03:	DS	31	;ALLOCATION VECTOR 3
CHK00:	DS	16	;CHECK VECTOR 0
CHK01:	DS	16	;CHECK VECTOR 1
CHK02:	DS	16	;CHECK VECTOR 2
CHK03:	DS	16	;CHECK VECTOR 3
;
ENDDAT	EQU	$	;END OF DATA AREA
DATSIZ	EQU	$-BEGDAT;SIZE OF DATA AREA
;
	END
